
source /usr/share/libubox/jshn.sh


#--------------------------------------------------------------
install_bin() { # <file> [ <symlink> ... ]
	echo "install_bin NOT USED, CAN LIKELY BE REMOVED! OR?"
	sleep 9999h
}



#--------------------------------------------------------------
v() {
	[ "$VERBOSE" -ge 1 ] && echo "$@"
}



#--------------------------------------------------------------
get_image() { # <source> [ <command> ]
	local from="$1"
	local conc="$2"
	local cmd

	local sysinfo=$(ubus call router.system info)
	json_load "$sysinfo"
	json_select system
	json_get_var firmware firmware
	json_get_var filesystem filesystem
	json_select ..
	json_cleanup

	case "$from" in
		http://*|ftp://*) cmd="wget -O- -q --user-agent=\"$firmware:$filesystem\"";;
		*) cmd="cat";;
	esac
	if [ -z "$conc" ]; then
		local magic="$(eval $cmd \"$from\" 2>/dev/null | dd bs=2 count=1 2>/dev/null | hexdump -n 2 -e '1/1 "%02x"')"
		case "$magic" in
			1f8b) conc="zcat";;
			425a) conc="bzcat";;
		esac
	fi

	eval "$cmd \"$from\" 2>/dev/null ${conc:+| $conc}"
}



#--------------------------------------------------------------
is_inteno_image() {
	local magic
	magic=$(head -c 10 "$1" | strings)
	[ "$magic" = "IntenoBlob" ] && return 0										# Magic for .y3
	return 1
}



#--------------------------------------------------------------
get_image_type() {
	if is_inteno_image $1; then
		echo "INTENO"
	else
		echo "UNKNOWN"
	fi
}



#--------------------------------------------------------------
get_inteno_tag_val() {
	local from="$1"
	local tag="$2"
	local val

	[ -r "$from" ] && val=$(head -c 1024 $from | strings | \
		awk "/^$tag / {print \$2}")
	[ -z "$val" ] && val=0
	echo $val
}



#--------------------------------------------------------------
get_image_board_id() {
	get_inteno_tag_val $1 board
}



#--------------------------------------------------------------
get_image_model_name() {
	get_inteno_tag_val $1 model
}



#--------------------------------------------------------------
get_image_customer() {
	get_inteno_tag_val $1 customer
}



#--------------------------------------------------------------
get_image_chip_id() {
	get_inteno_tag_val $1 chip
}



#--------------------------------------------------------------
# Returns byte size of a .y2 named section
# where image header is of version 3.
get_section_size_v3() {
	local from="$1"																# Image file
	local sec="$2"																# Named section
	local sz

	sz="$(get_inteno_tag_val $from $sec)"
	[ -z "$sz" ] && sz=0
	echo -n $sz
}



#--------------------------------------------------------------
# Returns byte offset in a .y2 image to a named section
# where image header is of version 3.
get_section_offset_v3() {
	local cfe_ofs cfe_sz k_ofs k_sz ubifs_ofs ubifs_sz
	local ubi_ofs ubi_sz pkg_ofs pkg_sz
	local from="$1"																# Image file
	local sec="$2"																# Named section

	cfe_ofs=1024
	cfe_sz=$(get_section_size_v3 $from cfe)
	k_ofs=$((cfe_ofs + cfe_sz))
	k_sz=$(get_section_size_v3 $from vmlinux)
	ubifs_ofs=$((k_ofs + k_sz))
	ubifs_sz=$(get_section_size_v3 $from ubifs)
	ubi_ofs=$((ubifs_ofs + ubifs_sz))
	ubi_sz=$(get_section_size_v3 $from ubi)
	pkg_ofs=$((ubi_ofs + ubi_sz))
	pkg_sz=$(get_section_size_v3 $from pkginfo)

	if [ "$sec" = "cfe" ]; then
		echo -n ${cfe_ofs}
	elif [ "$sec" = "vmlinux" ]; then
		echo -n ${k_ofs}
	elif [ "$sec" = "ubifs" ]; then
		echo -n ${ubifs_ofs}
	elif [ "$sec" = "ubi" ]; then
		echo -n ${ubi_ofs}
	elif [ "$sec" = "pkginfo" ]; then
		echo -n ${pkg_ofs}
	else
		echo -n 0
	fi
}



#--------------------------------------------------------------
# Returns byte offset in a .y3 image to a named section
# where image header is of version 5.
get_section_offset_v5() {
	local ofs filesize
	local from="$1"																# Image file
	local sec="$2"																# Named section

	ofs="$(get_inteno_tag_val $from ${sec}ofs)"
	filesize="$(get_inteno_tag_val $from size)"
	[ -z "$ofs" ] && ofs=0
	[ -z "$filesize" ] && filesize=0
	[ $ofs -ge 1024 -a $ofs -lt $filesize ] || ofs=0							# Verify tag is less than file size
	echo -n $ofs
}



#--------------------------------------------------------------
# Returns byte size of a .y2/.y3 image named section.
get_section_size() {
	local from="$1"																# Image file
	local ver

	ver=$(get_inteno_tag_val "$from" version)
	if [ $ver -ge 3 -a $ver -le 6 ]; then										# Version 4 differs from 3 in having a larger cfe.
		get_section_size_v3 "$@" || return										# Version 5 appear in .y3 and have section offsets.
	else
		echo -n 0
	fi
}



#--------------------------------------------------------------
# Returns byte offset in a .y2/.y3 image to a named section.
get_section_offset() {
	local from="$1"																# Image file
	local ver

	ver=$(get_inteno_tag_val "$from" version)
	if [ $ver -ge 3 -a $ver -le 4 ]; then										# Version 4 differs from 3 in having a larger cfe.
		get_section_offset_v3 "$@" || return
	elif [ $ver -ge 5 -a $ver -le 6 ]; then										# Version 5 appear in .y3 and have section offsets.
		get_section_offset_v5 "$@" || return
	else
		echo -n 0
	fi
}



#--------------------------------------------------------------
# Find which flash "bank" is currently used to boot
# from (either 0 or 1). Uses only shell built in
# functions for speed.
get_flashbank_current() {
	local cmdline

	read -t 5 -s cmdline <"/proc/cmdline"

	for opt in $cmdline; do
		case $opt in
			root=ubi*rootfs_0)
				echo -n 0
				;;
			root=ubi*rootfs_1)
				echo -n 1
				;;
		esac
	done
}



#--------------------------------------------------------------
# Find which flash "bank" is pending for upgrade.
get_flashbank_next() {
	local curr
	curr=$(get_flashbank_current)
	echo $(( (curr + 1) % 2))
}



#--------------------------------------------------------------
check_crc() {
	local from=$1
	local file_sz calc csum

	case $(get_inteno_tag_val $from integrity) in
		MD5SUM)
			file_sz=$(ls -l $from |awk '{print $5}')
			calc=$(head -c $(($file_sz-32)) $from |md5sum |awk '{print $1}')
			csum=$(tail -c 32 $from)
			[ "$calc" == "$csum" ] && echo "CRC_OK" || echo "CRC_BAD"
			;;
		*)
			echo "UNKNOWN"
			;;
	esac
}



#--------------------------------------------------------------
check_sig() {
	local from=$1
	local len

	len=$(($(get_section_size $from cfe) +
		$(get_section_size $from vmlinux) +
		$(get_section_size $from ubifs) +
		$(get_section_size $from ubi) ))

	# get pubkey from cert.
	openssl x509 -in /etc/ssl/certs/opkg.pem -pubkey -noout >/tmp/pubkey

	# extract signature data from firmware image.
	dd if=$from bs=1 skip=$((1024+len)) count=256 2>/dev/null >/tmp/sig

	result=$(dd if=$from bs=1024 skip=1 2>/dev/null | \
		head -c $len | \
		openssl dgst -sha256 \
			-verify /tmp/pubkey \
			-signature /tmp/sig )
	rm /tmp/pubkey
	rm /tmp/sig

	[ "$result" == "Verified OK" ] && return 0
	return 1
}



#--------------------------------------------------------------
# Verify an Iopsys image is good for writing
iopsys_check_image() {
	local cfe_sz k_sz ubi_sz ubifs_sz
	local from ver filesize

	if [ "$ARGC" -ne 1 ]; then
		echo "Wrong number of arguments"
		return 1
	fi

	echo "Image platform check started..."

	case "$1" in
		http://*|ftp://*)
			get_image "$1" "cat" >/tmp/firmware.bin
			from=/tmp/firmware.bin
			;;
		*)
			if [ -r "$1" ]; then
				from=$1
			else
				echo "No such file $1"
				return 1
			fi
			;;
	esac

	[ "$(check_crc $from)" == "CRC_OK" ] || {
		echo "CRC check failed"
		return 1
	}

	if [ -e /etc/ssl/certs/opkg.pem ]; then
		if ! check_sig "$from"; then
		echo "Signature of file is wrong. Aborting!"
		return 1
		fi
	fi

	if [ "$(get_image_chip_id "$from")" != "$(get_chip_id)" ]; then
		echo "Chip model of image does not match"
		return 1
	fi

	# Customer name check should be carried out
	# only if a regarding parameter set in config.
	# For now skip customer name check.
	if [ 1 -eq 0 ]; then
		[ -f /lib/db/version/iop_customer ] \
			&& [ "$(get_image_customer "$from")" != "$(cat /lib/db/version/iop_customer)" ] && {
			echo "Image customer doesn't match"
			return 1
		}
		# NOTE: expr interprets $(db get hw.board.hardware) as a
		# regexp which could give unexpected results if the harware
		# name contains any magic characters.
		expr "$(get_image_model_name "$from")" : "$(db get hw.board.hardware)" || {
			echo "Image model name doesn't match board hardware"
			return 1
		}
	fi

	# Verify model
	model=$(get_image_model_name "$from")
	hwmodel=$(db get hw.board.hardware)
	if [ $model != $hwmodel ]; then
		echo "HW model: |$hwmodel| - Model: |$model| check failed!"
		return 1
	fi

	# Sanity check payload section sizes
	cfe_sz=$(get_section_size $from cfe) || return 1
	k_sz=$(get_section_size $from vmlinux) || return 1
	ubi_sz=$(get_section_size $from ubi) || return 1
	ubifs_sz=$(get_section_size $from ubifs) || return 1
	ver=$(get_inteno_tag_val "$from" version) || return 1
	        if [ $ver -eq 3 ]; then
                echo "y2 image unsupported"
                return 1
        fi
	filesize=$(get_inteno_tag_val "$from" size) || return 1
	if [ $ver -le 4 -a $cfe_sz -eq 0 -a $k_sz -eq 0 -a $ubi_sz -eq 0 \
			-a $ubifs_sz -eq 0 ]; then
		echo "Unexpected image file contents, upgrade aborted!"
		return 1
	elif [ $ver -le 4 -a  $((cfe_sz + k_sz + ubi_sz + ubifs_sz)) -gt \
			$(ls -l $from | awk '{print $5}') ]; then
		echo "Image file too small, upgrade aborted!"
		return 1
	elif [ $ver -ge 5 -a $filesize -ne $(ls -l $from | awk '{print $5}') ]; then
		echo "Invalid file size, upgrade aborted!"
		return 1
	fi


	echo "Image platform check completed."

	return 0
}



#--------------------------------------------------------------
# Upgrade system with an Iopsys image
iopsys_upgrade() {
	local from mtd_no
	local klink sname

	case "$1" in
		http://*|ftp://*) from=/tmp/firmware.bin;;
		*) from=$1;;
	esac


	# Stop unnecessary processes
	for klink in $(ls /etc/rc.d/K[0-9]*); do
		sname=$(basename $(readlink -f $klink))
		case $sname in
			boot|cgroups|dropbear|icwmpd|inbd|iwatchdog|telnet|umount)
				v "  Not stopping $sname"
				;;
			*)
				v "  Stopping $sname..."
				$klink stop
				;;
		esac
	done
	sleep 3s

	if [ "$SAVE_OVERLAY" -eq 1 ]; then
		v "Creating save overlay file marker."
		touch /SAVE_OVERLAY
	else
		v "Not saving overlay files."
		rm -f /SAVE_OVERLAY
	fi
	if [ "$SAVE_CONFIG" -eq 1 ]; then
		v "Creating save config file marker."
		touch /SAVE_CONFIG
	else
		v "Not saving config files."
		rm -f /SAVE_CONFIG
	fi

	# Product specific low level write to flash
	target_upgrade $from || return
}


